/*

DashMail.js

Copyright (c) 2006 Scue

www.scue.co.uk (www.scue.co.uk/widgets/)
info@scue.co.uk
developer: aidan kane (aidan@scue.co.uk)

see LICENSE for details of the license

*/


var dashMail = null;

// called when the widget first loads - just creates an instance of DashMail
// the DashMail object then does the rest of the work itself
function init() {
	dashMail = new DashMail();
	dashMail.init();
}

var debugEnabled = true;

function debug(text) {
    if (debugEnabled && window.widget) {
        alert(text);
    }
}

// DashMail is really the main running programme
// an instance is created when this widget first loads and
// all events are sent to it to be redistributed to the other objects
function DashMail()
{
	// user details
	this.userEmail = '';
	this.password = '';
	this.mailHost = '';
	this.otherMailHost = '';

	// helper objects
	this.display = null;
	this.mailSender = null;
	this.persister = null;
}

DashMail.prototype =
{

	// initial setup of the widget
	// - creates the objects/elements used
	// - retrieves stored user details
	// - starts the mail checking process
	init : function()
	{
		var localThis = this;

		// object to allow us to save/load user details
		this.persister = new Persister();
		this.persister.setApplicationName('DashMail');
		// retrieve saved configuration
		this.userEmail = this.persister.getPref('UserEmail');
		this.password = this.persister.getPassword(this.userEmail);
		this.mailHost = this.persister.getPref('MailHost');
		this.username = this.persister.getPref('Username');
		this.otherMailHost = this.persister.getPref('OtherMailHost');
		this.port = this.persister.getPref('Port');
		this.useSSL = this.persister.getPref('UseSSL');
		var storedRecipients = this.persister.getListPref('Recipient');
		var storedSubjects = this.persister.getListPref('Subject');

		// this object is used to access all the interface properties
		this.display = new Display();
		this.display.init(function() {localThis.clickDoneButton();},storedRecipients,storedSubjects);

		// push the email/password etc to the interface
		this.display.setUserEmail(this.userEmail);
		this.display.setPassword(this.password);
		if (this.mailHost == '')
		{
			this.mailHost = this.display.getMailHost();
		} else {
			this.display.setMailHost(this.mailHost);
		}
		this.display.setUsername(this.username);
		this.display.setOtherMailHost(this.otherMailHost);
		this.display.setPort(this.port);
		this.display.setUseSSL(this.useSSL);
		this.changeMailHost();

		// set the event handlers for the widget
		this.setupWidgetEventHandlers();

		// flip over to prefs if we don't have a email/password
		if (!this.loginValid())
		{
			// todo: turn this back on
			setTimeout(function() {localThis.gotoPrefs();}, 500);
		}
	},

	setupWidgetEventHandlers : function()
	{
		var localThis = this;

		// attach our functions for the events
		if (window.widget)
		{
			widget.onload = function()
			{
				window.resizeTo(244,264);
			}

			widget.onremove = function()
			{
				debug("widget removed");
				// clear preferences
				localThis.persister.destroyPreferences();
			}
		}
	},

	sendMessage : function()
	{
		debug('sending message');
		
		var localThis = this;

		this.display.startProgressIndicator();

		// object that handles the sending of the email
		this.mailSender = new MailSender();
		var localThis = this;
		this.mailSender.init(function (result) {localThis.finishedMessageSend(result);});

		this.mailSender.serverType = this.mailHost;
		this.mailSender.fromAddress = this.userEmail;
		this.mailSender.username = this.username;
		this.mailSender.password = this.password;
		this.mailSender.mailHost = this.otherMailHost;
		this.mailSender.port = this.port;
		this.mailSender.useSSL = this.useSSL;

		this.mailSender.recipient = this.display.getRecipient();
		this.mailSender.subject = this.display.getSubject();
		this.mailSender.message = this.display.getMessage();

		this.mailSender.sendMessage();
	},

	// called after finished sending the message
	finishedMessageSend : function(sendOk)
	{
		this.display.stopProgressIndicator();

		if (sendOk)
		{
			debug("message sent ok");

			// display result to user
			var dateNow = new Date();
			var hours = dateNow.getHours();
			var minutes = dateNow.getMinutes();
			var amPM = "am";

			if (minutes < 10) minutes = "0" + minutes
			if (hours > 11) amPM = "pm";
			if (hours > 12) hours = hours - 12;

			var currentTime = hours + ":" + minutes + amPM;

			this.display.showResult("sent at " + currentTime);

			// update the dropdown lists
			this.display.addValuesToDropdowns();
			this.persister.storeListPref('Recipient',this.display.recipientList.getList());
			this.persister.storeListPref('Subject',this.display.subjectList.getList());
		}
		else
		{
			// let the user know that something went wrong
			this.display.showResult("couldn't send!");
		}
	},

	gotoPrefs : function()
	{
		this.display.flipToBack();
	},

	savePreferencesAndFlipToFront : function()
	{
		var newuseremail = this.display.getUserEmail();
		var newpassword = this.display.getPassword();

		if (newuseremail != '' && (newuseremail != this.userEmail || newpassword != this.password))
		{
			// save the password in the keychain
			this.persister.storePref('UserEmail',newuseremail);
			this.persister.storePassword(newuseremail,newpassword);
		}

		this.userEmail = newuseremail;
		this.password = newpassword;
		this.username = this.display.getUsername();
		this.otherMailHost = this.display.getOtherMailHost();
		this.port = this.display.getPort();
		this.useSSL = this.display.getUseSSL();

		this.mailHost = this.display.getMailHost();
		this.persister.storePref('MailHost',this.mailHost);
		this.persister.storePref('Username',this.username);
		this.persister.storePref('OtherMailHost',this.otherMailHost);
		this.persister.storePref('Port',this.port);
		this.persister.storePref('UseSSL',this.useSSL);

		if (this.loginValid())
		{
			// todo: maybe we should try connecting to the server now?
		}

		this.display.flipToFront();
		this.display.unhighlightFlipButton();
	},

	loginValid : function()
	{
		var userEmailValid = (this.userEmail && (this.userEmail.length > 0));
		var passwordValid = (this.password && (this.password.length > 0));

		return (userEmailValid && passwordValid);
	},


	/*

	Functions to handle events that occur on the widget interface

	*/

	changeRecipient : function()
	{
		this.display.changeRecipient();
		this.persister.storeListPref('Recipient',this.display.recipientList.getList());
	},

	changeSubject : function()
	{
		this.display.changeSubject();
		this.persister.storeListPref('Subject',this.display.subjectList.getList());
	},

	clickSend : function()
	{
		this.display.hideResult();
		this.sendMessage();
	},

	// save prefs and flip to the front when they hit 'enter' on prefs
	keyPressOnPrefs : function(event) {
		var keyCode = event.keyCode ? event.keyCode : event.which ? event.which : event.charCode;
		if (keyCode == 13) {
			this.savePreferencesAndFlipToFront();
		}
	},

	// save prefs, flip to the front and check for new mail immediately
	clickDoneButton : function() {
		this.savePreferencesAndFlipToFront();
	},

	mouseOverFront : function()
	{
//		this.display.showFlipButton();
	},

	mouseOutFront : function()
	{
//		this.display.hideFlipButton();
	},

	mouseOverFlip : function()
	{
		this.display.highlightFlipButton();
	},

	mouseOutFlip : function()
	{
		this.display.unhighlightFlipButton();
	},

	mouseClickFlip : function()
	{
		this.gotoPrefs();
	},

	focusSubject : function(event)
	{
		var localThis = this;
		setTimeout(function(){localThis.display.selectSubject();},100);
		this.display.highlightSubject();
	},

	blurSubject : function(event)
	{
		this.display.unhighlightSubject();
		this.subjectFirstClick = true;
	},

	focusRecipient : function(event)
	{
		var localThis = this;
		setTimeout(function(){localThis.display.selectRecipient();},100);
		this.display.highlightRecipient();
	},

	blurRecipient : function(event)
	{
		this.display.unhighlightRecipient();
		this.recipientFirstClick = true;
	},

	focusMessage : function(event)
	{
		var localThis = this;
		setTimeout(function(){localThis.display.selectMessage();},100);
		this.display.highlightMessage();
	},

	blurMessage : function(event)
	{
		this.display.unhighlightMessage();
		this.messageBodyFirstClick = true;
	},	

	keyDownRecipient : function(event)
	{
		switch (event.keyCode)
		{
			// tab to next control
			case 9 :
				document.getElementById('subject').focus();
				return false;

/*			// shift tab to previous control
			case 0 :
				document.getElementById('messageBody').focus();			
				return false;
*/
			// up/down = display dropdown
			// todo: make this display the dropdown
			case 38 :
				document.getElementById('realRecipient').focus();
				return false;
			case 40 :
				document.getElementById('realRecipient').focus();
				return false;
		}

		return true;
	},

	keyDownSubject : function(event)
	{
		switch (event.keyCode)
		{
			// tab to next control
			case 9 :
				document.getElementById('messageBody').focus();
				return false;

			// shift tab to previous control
/*			case 0 :
				document.getElementById('recipient').focus();
				return false;
*/
			// up/down = display dropdown
			// todo: make this display the dropdown
			case 38 :
				document.getElementById('realSubject').focus();
				return false;
			case 40 :
				document.getElementById('realSubject').focus();
				return false;
		}

		return true;
	},

	keyDownMessage : function(event)
	{
		switch (event.keyCode)
		{
			// tab to next control
			case 9 :
				document.getElementById('recipient').focus();
				return false;

/*			// shift tab to previous control
			case 0 :
				document.getElementById('subject').focus();
				return false;
*/		}

		return true;
	},
	
	changeMailHost : function()
	{
		if (this.display.getMailHost() == "other")
		{
			this.display.showExtraOptions(true);
		} else {
			this.display.showExtraOptions(false);
		}
	},
	
	openScueSite : function()
	{
		if (window.widget)
		{
			widget.openURL("http://www.scue.co.uk/widgets/");
		}
	}
}
